﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.BusinessLogicContracts;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreBusinessLogic.BusinessLogic
{
    public class ReportOrderLogic : IReportOrderLogic
    {
        private readonly IOrderStorage _orderStorage;
        private readonly IConsignmentStorage _consignmentStorage;
        private readonly IRequestStorage _requestStorage;
        private readonly IPCStorage _pcStorage;

        public ReportOrderLogic(
            IOrderStorage orderStorage,
            IConsignmentStorage consignmentStorage,
            IRequestStorage requestStorage,
            IPCStorage pcStorage)
        {
            _orderStorage = orderStorage;
            _consignmentStorage = consignmentStorage;
            _requestStorage = requestStorage;
            _pcStorage = pcStorage;
        }

        public ReportOrderViewModel GetPCsByOrders(ReportOrderBindingModel model)
        {
            var report = new ReportOrderViewModel();
            List<OrderViewModel> orders = new List<OrderViewModel>();

            foreach (var orderSearch in model.Orders)
            {
                var element = _orderStorage.GetElement(orderSearch);
                if (element == null)
                {
                    throw new Exception("Selected order is not find.");
                }
                orders.Add(element);
            }

            foreach (var order in orders)
            {
                if (order.OrderRequests == null) { continue; }
                report.OrderPCs.Add(order.ID, (new(), new()));

                foreach (var requestRaw in order.OrderRequests)
                {
                    var request = _requestStorage.GetElement(new RequestSearchModel { ID = requestRaw.ID });
                    if (request == null) { throw new Exception("Request for report is not find."); }

                    var pc = _pcStorage.GetElement(new PCSearchModel { ID = request.PCID });
                    if (pc == null) { throw new Exception("PC for report is not find."); }

                    report.OrderPCs[order.ID].Item1.Add(request);
                    report.OrderPCs[order.ID].Item2.Add(pc);
                }
            }
            return report;
        }

        public ReportOrderByDateViewModel GetOrdersByDate(ReportDateBindingModel model)
        {
            var report = new ReportOrderByDateViewModel();
            var orders = _orderStorage.GetFullList();
            foreach (var order in orders)
            {
                if (order.OrderConsignments == null && order.OrderRequests == null) { continue; }

                if (order.DateCreate >= model.DateFrom && order.DateCreate <= model.DateTo)
                {
                    report.OrderConsignmentsAndRequests.Add(order.ID, (new(), new()));

                    if (order.OrderConsignments != null)
                    {
                        foreach (var consignmentRaw in order.OrderConsignments)
                        {
                            var consignment = _consignmentStorage.GetElement(new ConsignmentSearchModel { ID = consignmentRaw.ID });
                            if (consignment == null)
                            {
                                throw new Exception("Consignment for create report is not found");
                            }
                            report.OrderConsignmentsAndRequests[order.ID].Item1.Add(consignment);
                        }
                    }

                    if (order.OrderRequests != null)
                    {
                        foreach (var requestRaw in order.OrderRequests)
                        {
                            var request = _requestStorage.GetElement(new RequestSearchModel { ID = requestRaw.ID });
                            if (request == null)
                            {
                                throw new Exception("Request for create report is not found");
                            }
                            report.OrderConsignmentsAndRequests[order.ID].Item2.Add(request);
                        }
                    }
                }
            }
            return report;
        }

        public void SaveOrdersToExcelFile(ReportOrderBindingModel model)
        {
            throw new NotImplementedException();
        }

        public void SaveOrdersToPdfFile(ReportDateBindingModel model)
        {
            throw new NotImplementedException();
        }

        public void SaveOrdersToWordFile(ReportOrderBindingModel model)
        {
            throw new NotImplementedException();
        }
    }
}
