﻿using ComputerStoreBusinessLogic.OfficePackage.HelperEnums;
using ComputerStoreBusinessLogic.OfficePackage.HelperModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreBusinessLogic.OfficePackage
{
    public abstract class AbstractSaveToWord
    {
        public MemoryStream CreateDoc(WordInfo info)
        {
            CreateWord(info);
            CreateParagraph(new WordParagraph
            {
                Texts = new List<(string, WordTextProperties)> { (info.Title, new WordTextProperties { Bold = true, Size = "24", }) },
                TextProperties = new WordTextProperties
                {
                    Size = "24",
                    JustificationType = WordJustificationType.Center
                }
            });
            foreach (var consigment in info.Consignments)
            {
                CreateParagraph(new WordParagraph
                {
                    Texts = new List<(string, WordTextProperties)> { ($"ConsignmentID:  {consigment.ID}", new WordTextProperties { Bold = true, Size = "24", }) },
                    TextProperties = new WordTextProperties
                    {
                        Size = "24",
                        JustificationType = WordJustificationType.Both
                    }
                });

                foreach(var products in consigment.Products)
                {
                    CreateParagraph(new WordParagraph
                    {
                        Texts = new List<(string, WordTextProperties)> { 
                            ($"ProductID:  {products.product}     ", new WordTextProperties { Bold = false, Size = "24", }), ($"ProductCount:  {products.count}", new WordTextProperties { Bold = false, Size = "24", }) },
                        TextProperties = new WordTextProperties
                        {
                            Size = "24",
                            JustificationType = WordJustificationType.Both
                        }
                    });
                }

            }
            return SaveWord(info);
        }

        protected abstract void CreateWord(WordInfo info);
        protected abstract void CreateParagraph(WordParagraph paragraph);
        protected abstract MemoryStream SaveWord(WordInfo info);
    }
}
