﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreDatabaseImplement.Implements
{
    public class ComponentStorage : IComponentStorage
    {
        
        public ComponentViewModel? GetElement(ComponentSearchModel model)
        {
            if(string.IsNullOrEmpty(model.Name) && !model.ID.HasValue)
            {
                return null;
            }
            using var context = new ComputerStoreDatabase();
            return context.Components.FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && model.Name == x.Name) || (model.ID.HasValue && model.ID ==x.ID))?.GetViewModel;
        }

        public List<ComponentViewModel> GetFilteredList(ComponentSearchModel model)
        {
            if(string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            using var context = new ComputerStoreDatabase();
            return context.Components.Where(x => x.Name.Equals(model.Name)).Select(x => x.GetViewModel).ToList();
        }

        public List<ComponentViewModel> GetFullList()
        {
            using var context = new ComputerStoreDatabase();
            return context.Components.Select(x => x.GetViewModel).ToList();
        }

        public ComponentViewModel? Insert(ComponentBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var newComponent = Component.Create(model);

            if(newComponent == null) { return null; }

            context.Components.Add(newComponent);
            context.SaveChanges();
            return newComponent.GetViewModel;
        }

        public ComponentViewModel? Update(ComponentBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var specComponent = context.Components.FirstOrDefault(x => x.ID == model.ID);

            if(specComponent == null) { return null; }

            specComponent.Update(model);
            context.SaveChanges();
            return specComponent.GetViewModel;
        }

        public ComponentViewModel? Delete(ComponentBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var specComponent = context.Components.FirstOrDefault(x => x.ID==model.ID);

            if(specComponent == null) { return null; }

            context.Components.Remove(specComponent);
            context.SaveChanges();
            return specComponent.GetViewModel;  
        }
    }
}
