﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace ComputerStoreDatabaseImplement.Implements
{
    public class ConsignmentStorage : IConsignmentStorage
    {
        public List<ConsignmentViewModel> GetFullList()
        {
            using var context = new ComputerStoreDatabase();
            return context.Consignments
                                   .Include(x => x.Order)
                                   .Include(x => x.Products)
                                   .ThenInclude(x => x.Product)
                                   .Select(x => x.GetViewModel)
                                   .ToList();
        }

        public List<ConsignmentViewModel> GetFilteredList(ConsignmentSearchModel model)
        {
            using var context = new ComputerStoreDatabase();
            return context.Consignments.Include(x => x.Order)
                                   .Include(x => x.Products)
                                   .ThenInclude(x => x.Product)
                                   .Where(x => 
                                   (!model.ID.HasValue || x.ID == model.ID) && 
                                   (!model.OrderID.HasValue || x.OrderID == model.OrderID) &&
                                   (!model.ProductID.HasValue || x.Products.FirstOrDefault(y => y.ProductID == model.ProductID) != null))
                                   .Select(x => x.GetViewModel)
                                   .ToList();
        }

        public ConsignmentViewModel? GetElement(ConsignmentSearchModel model)
        {
            using var context = new ComputerStoreDatabase();
            return context.Consignments.Include(x => x.Order)
                                   .Include(x => x.Products)
                                   .ThenInclude(x => x.Product)
                                   .FirstOrDefault(x => 
                                   (!model.ID.HasValue || x.ID == model.ID) && 
                                   (!model.OrderID.HasValue || x.OrderID == model.OrderID) &&
                                   (!model.ProductID.HasValue || x.Products.FirstOrDefault(y => y.ProductID == model.ProductID) != null))?
                                   .GetViewModel;
        }

        public ConsignmentViewModel? Insert(ConsignmentBindingModel model)
        {
            using var context = new ComputerStoreDatabase();

            var newConsignment = Consignment.Create(context, model);
            if (newConsignment == null)
            {
                return null;
            }

            context.Consignments.Add(newConsignment);
            context.SaveChanges();

            return context.Consignments.Include(x => x.Order)
                                   .Include(x => x.Products)
                                   .ThenInclude(x => x.Product)
                                   .FirstOrDefault(x => x.ID == newConsignment.ID)?
                                   .GetViewModel;
        }

        public ConsignmentViewModel? Update(ConsignmentBindingModel model)
        {
            using var context = new ComputerStoreDatabase();

            var consignment = context.Consignments.Include(x => x.Products).FirstOrDefault(x => x.ID == model.ID);
            if (consignment == null)
            {
                return null;
            }

            consignment.Update(model);
            consignment.UpdateProducts(context, model);
            context.SaveChanges();

            return context.Consignments.Include(x => x.Order)
                                   .Include(x => x.Products)
                                   .ThenInclude(x => x.Product)
                                   .FirstOrDefault(x => x.ID == consignment.ID)?
                                   .GetViewModel;
        }

        public ConsignmentViewModel? Delete(ConsignmentBindingModel model)
        {
            using var context = new ComputerStoreDatabase();

            var consignment = context.Consignments.Include(x => x.Order)
                                   .Include(x => x.Products)
                                   .ThenInclude(x => x.Product)
                                   .FirstOrDefault(x => x.ID == model.ID);
            if (consignment != null)
            {
                context.Consignments.Remove(consignment);
                context.SaveChanges();

                return consignment.GetViewModel;
            }

            return null;
        }
    }
}
