﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreDatabaseImplement.Implements
{
    public class EmployeeStorage : IEmployeeStorage
    {

        public EmployeeViewModel? GetElement(EmployeeSearchModel model)
        {
            if(string.IsNullOrEmpty(model.Username) && !model.ID.HasValue)
            {
                return null;
            }
            using var context = new ComputerStoreDatabase();
            return context.Employees.FirstOrDefault(x => (!string.IsNullOrEmpty(model.Username) && model.Username == x.Username) || (model.ID.HasValue && model.ID == x.ID))?.GetViewModel;
        }

        public List<EmployeeViewModel> GetFilteredList(EmployeeSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Username))
            {
                return new();
            }
            using var context = new ComputerStoreDatabase();
            return context.Employees.Where(x => x.Username.Equals(model.Username)).Select(x => x.GetViewModel).ToList();
        }

        public List<EmployeeViewModel> GetFullList()
        {
            using var context = new ComputerStoreDatabase();
            return context.Employees.Select(x => x.GetViewModel).ToList();
        }

        public EmployeeViewModel? Insert(EmployeeBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var newEmployee = Employee.Create(model);

            if(newEmployee == null) { return null; }
        
            context.Employees.Add(newEmployee);
            context.SaveChanges();
            return newEmployee.GetViewModel;
        }

        public EmployeeViewModel? Update(EmployeeBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var specEmployee = context.Employees.FirstOrDefault(x => x.ID == model.ID);

            if (specEmployee == null) { return null; }

            specEmployee.Update(model);
            context.SaveChanges();
            return specEmployee.GetViewModel;
        }

        public EmployeeViewModel? Delete(EmployeeBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var specEmployee = context.Employees.FirstOrDefault(x => x.ID == model.ID);

            if (specEmployee == null) { return null; }

            context.Employees.Remove(specEmployee);
            context.SaveChanges();
            return specEmployee.GetViewModel;
        }
    }
}
