﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new ComputerStoreDatabase();
            return context.Orders.Include(x => x.Seller)
                                   .Include(x => x._consignments)
                                   .Include(x => x._requests)
                                   .Select(x => x.GetViewModel)
                                   .ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new ComputerStoreDatabase();
            return context.Orders
                                .Include(x => x.Seller)
                                .Include(x => x._consignments)
                                .Include(x => x._requests)
                                .Where(x =>
                                (!model.ID.HasValue || 
                                x.ID == model.ID) &&
                                (!model.SellerID.HasValue || 
                                x.SellerID == model.SellerID) &&
                                (!model.ConsignmentID.HasValue || 
                                x._consignments.FirstOrDefault(y => y.ID == model.ConsignmentID) != null) &&
                                (!model.RequestID.HasValue || 
                                x._requests.FirstOrDefault(y => y.ID == model.RequestID) != null) &&
                                (!model.DateFrom.HasValue || x.DateCreate >= model.DateFrom) &&
                                (!model.DateTo.HasValue || x.DateCreate <= model.DateTo))
                                .Select(x => x.GetViewModel)
                                .ToList();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            using var context = new ComputerStoreDatabase();
            return context.Orders.Include(x => x.Seller)
                                .Include(x => x._consignments)
                                .Include(x => x._requests)
                                .FirstOrDefault(x =>
                                (!model.ID.HasValue ||
                                x.ID == model.ID) &&
                                (!model.SellerID.HasValue ||
                                x.SellerID == model.SellerID) &&
                                (!model.ConsignmentID.HasValue ||
                                x._consignments.FirstOrDefault(y => y.ID == model.ConsignmentID) != null) &&
                                (!model.RequestID.HasValue ||
                                x._requests.FirstOrDefault(y => y.ID == model.RequestID) != null) &&
                                (!model.DateFrom.HasValue || x.DateCreate >= model.DateFrom) &&
                                (!model.DateTo.HasValue || x.DateCreate <= model.DateTo))?
                                .GetViewModel;

        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new ComputerStoreDatabase();

            var newOrder = Order.Create(context, model);
            if (newOrder == null)
            {
                return null;
            }

            context.Orders.Add(newOrder);
            context.SaveChanges();

            return context.Orders.Include(x => x.Seller)
                                .Include(x => x._consignments)
                                .Include(x => x._requests)
                                .FirstOrDefault(x => x.ID == newOrder.ID)?
                                .GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new ComputerStoreDatabase();

            var order = context.Orders
                                .Include(x => x._consignments)
                                .Include(x => x._requests)
                                .FirstOrDefault(x => x.ID == model.ID);
            if (order == null)
            {
                return null;
            }

            order.Update(model);
            order.UpdateOrderConsignments(context, model);
            order.UpdateOrderRequests(context, model);
            context.SaveChanges();

            return context.Orders.Include(x => x.Seller)
                                .Include(x => x._consignments)
                                .Include(x => x._requests)
                                .FirstOrDefault(x => x.ID == model.ID)?
                                .GetViewModel;
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new ComputerStoreDatabase();

            var order = context.Orders.Include(x => x.Seller)
                                .Include(x => x._consignments)
                                .Include(x => x._requests)
                                .FirstOrDefault(x => x.ID == model.ID);
            if (order != null)
            {
                context.Orders.Remove(order);
                context.SaveChanges();

                return order.GetViewModel;
            }

            return null;
        }
    }
}
