﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Transactions;

namespace ComputerStoreDatabaseImplement.Implements
{
    public class PCStorage : IPCStorage
    {
       
        public PCViewModel? GetElement(PCSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.ID.HasValue) { return null; }
            
            using var context = new ComputerStoreDatabase();
            return context.PCS.Include(x => x.Employee).Include(x => x.Request).ThenInclude(x => x.PCs).ThenInclude(x => x.Component).FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && model.Name.Equals(x.Name)) || (model.ID.HasValue && model.ID == x.ID))?.GetViewModel;

        }

        public List<PCViewModel> GetFilteredList(PCSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.EmployeeID.HasValue && !model.ID.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue)
            {
                return new();
            }
            using var context = new ComputerStoreDatabase();
  
            if(model.EmployeeID.HasValue)
            {
                return context.PCS.Include(x => x.Employee).Include(x => x.Request).ThenInclude(x => x.PCs).ThenInclude(x => x.Component).Where(x => x.EmployeeID == model.EmployeeID).Select(x => x.GetViewModel).ToList();
            }
            return context.PCS.Include(x => x.Employee).Include(x => x.Request).ThenInclude(x => x.PCs).ThenInclude(x => x.Component).Where(p => context.Requests.Where(r => context.Orders.Where(o => o.DateCreate >= model.DateFrom && o.DateCreate <= model.DateTo).Select(o => o.ID).Contains(r.OrderID)).Select(r => r.ID).Contains(p.RequestID)).ToList().Select(x => x.GetViewModel).ToList();
        }

        public List<PCViewModel> GetFullList()
        {
            using var context = new ComputerStoreDatabase();
            return context.PCS.Include(x => x.Employee).Include(x => x.Request).ThenInclude(x => x.PCs).ThenInclude(x => x.Component).ToList().Select(x => x.GetViewModel).ToList();
        }

        public PCViewModel? Insert(PCBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var newPC = PC.Create(context, model);
            if(newPC == null) { return null; }

            context.PCS.Add(newPC);

            

            context.SaveChanges();

           

            var pc = context.PCS.FirstOrDefault(x => x.Name.Equals(model.Name));
            PC.EnterPCID(context, pc);

            return newPC.GetViewModel;
        }

        public PCViewModel? Update(PCBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var specPC = context.PCS.Include(x => x.Request).Include(x => x.Employee).FirstOrDefault(x => x.ID == model.ID);
                if(specPC == null) { return null; }

                specPC.Update(model);
                context.SaveChanges();
                specPC.UpdateComponents(context, model);
                transaction.Commit();
                return specPC.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public PCViewModel? Delete(PCBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var specPC = context.PCS.Include(x => x.Request).Include(x => x.Employee).FirstOrDefault(x => x.ID == model.ID);
            if(specPC == null) { return null;}
            context.PCS.Remove(specPC);
            context.SaveChanges();
            return specPC.GetViewModel; 
        }

    }
}
