﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreDatabaseImplement.Implements
{
    public class ProductStorage : IProductStorage
    {
        public ProductViewModel? GetElement(ProductSearchModel model)
        {
            if(string.IsNullOrEmpty(model.Name) && !model.ID.HasValue) { return null; }
            using var context = new ComputerStoreDatabase();

            return context.Products.Include(x => x.Components).ThenInclude(x => x.Component).FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && model.Name.Equals(x.Name)) || (model.ID.HasValue && model.ID == x.ID))?.GetViewModel;
        }

        public List<ProductViewModel> GetFilteredList(ProductSearchModel model)
        {
            if(string.IsNullOrEmpty(model.Name) && !model.EmployeeID.HasValue && !model.DateTo.HasValue && !model.DateFrom.HasValue && !model.ComponentID.HasValue)
            {
                return new();
            }

            using var context = new ComputerStoreDatabase();
            if(model.EmployeeID.HasValue)
            {
                return context.Products.Include(x => x.Employee).Where(x => x.EmployeeID == model.EmployeeID).Select(x => x.GetViewModel).ToList();
            }
            if(model.ComponentID.HasValue)
            {
                return context.Products.Include(x => x.Components).ThenInclude(x => x.Component).Where(p => context.ProductComponents.Where(pc => pc.ComponentID == model.ComponentID).Select(pc => pc.ProductID).Contains(p.ID)).Select(x => x.GetViewModel).ToList();
            }
            return context.Products.Include(x => x.Components).ThenInclude(x => x.Component).Where(p => context.ConsignmentProducts.Where(cp => context.Consignments.Where(c => context.Orders.Where(o => o.DateCreate >= model.DateFrom && o.DateCreate <= model.DateTo).Select(o => o.ID).Contains(c.OrderID)).Select(c => c.ID).Contains(cp.ConsignmentID)).Select(cp => cp.ProductID).Contains(p.ID)).ToList().Select(x => x.GetViewModel).ToList();
        }

        public List<ProductViewModel> GetFullList()
        {
            using var context = new ComputerStoreDatabase();
            return context.Products.Include(x => x.Components).ThenInclude(x => x.Component).ToList().Select(x => x.GetViewModel).ToList();
        }

        public ProductViewModel? Insert(ProductBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var newProduct = Product.Create(context, model);

            if(newProduct == null) { return null; }

            context.Products.Add(newProduct);
            context.SaveChanges();
            return newProduct.GetViewModel;
        }

        public ProductViewModel? Update(ProductBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var specProduct = context.Products.FirstOrDefault(x => x.ID==model.ID);
                if(specProduct == null) { return null; }
                specProduct.Update(model);
                context.SaveChanges();
                specProduct.UpdateComponents(context, model);
                transaction.Commit();
                return specProduct.GetViewModel; 
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public ProductViewModel? Delete(ProductBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var specProduct = context.Products.Include(x => x.Components).FirstOrDefault(x => x.ID == model.ID);
            if(specProduct == null) { return null; }

            context.Products.Remove(specProduct);
            context.SaveChanges();
            return specProduct.GetViewModel;    
        }
    }
}
