﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreDatabaseImplement.Implements
{
    public class SellerStorage : ISellerStorage
    {
        public SellerViewModel? GetElement(SellerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Username) && !model.ID.HasValue)
            {
                return null;
            }
            using var context = new ComputerStoreDatabase();
            return context.Sellers.FirstOrDefault(x => (!string.IsNullOrEmpty(model.Username) && model.Username == x.Username) || (model.ID.HasValue && model.ID == x.ID))?.GetViewModel;
        }

        public List<SellerViewModel> GetFilteredList(SellerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Username))
            {
                return new();
            }
            using var context = new ComputerStoreDatabase();
            return context.Sellers.Where(x => x.Username.Equals(model.Username)).Select(x => x.GetViewModel).ToList();
        }

        public List<SellerViewModel> GetFullList()
        {
            using var context = new ComputerStoreDatabase();
            return context.Sellers.Select(x => x.GetViewModel).ToList();
        }

        public SellerViewModel? Insert(SellerBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var newSeller = Seller.Create(model);

            if (newSeller == null) { return null; }

            context.Sellers.Add(newSeller);
            context.SaveChanges();
            return newSeller.GetViewModel;
        }

        public SellerViewModel? Update(SellerBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var specSeller = context.Sellers.FirstOrDefault(x => x.ID == model.ID);

            if (specSeller == null) { return null; }

            specSeller.Update(model);
            context.SaveChanges();
            return specSeller.GetViewModel;
        }

        public SellerViewModel? Delete(SellerBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var specSeller = context.Sellers.FirstOrDefault(x => x.ID == model.ID);

            if (specSeller == null) { return null; }

            context.Sellers.Remove(specSeller);
            context.SaveChanges();
            return specSeller.GetViewModel;
        }
    }
}
