﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreDatabaseImplement.Models
{
    public class Product : IProductModel
    {
        public int ID { get; private set; }

        [Required]
        public string Name { get; private set; } = string.Empty;

        [Required]
        public double Price { get; private set; }

        [Required]
        public int EmployeeID { get; private set; }


        private Dictionary<int, (IComponentModel, int)>? _productComponents = null;

        [NotMapped]
        public Dictionary<int, (IComponentModel, int)> ProductComponents
        {
            get
            {
                if(_productComponents == null)
                {
                    _productComponents = Components.ToDictionary(recPC => recPC.ComponentID, recPC => (recPC.Component as IComponentModel, recPC.Count));
                }
                return _productComponents;
            }
        }

        [ForeignKey("ProductID")]
        public virtual List<ProductComponent> Components { get; set; } = new();

        [ForeignKey("ProductID")]
        public virtual List<ConsignmentProduct> Consignments { get; set; } = new();
       
        public virtual Employee Employee { get; set; }
        public static Product Create(ComputerStoreDatabase context, ProductBindingModel model)
        {
            return new Product()
            {
                ID = model.ID,
                Name = model.Name,
                Price = model.Price,
                EmployeeID = model.EmployeeID,
                Components = model.ProductComponents.Select(x => new ProductComponent
                {
                    Component = context.Components.First(y => y.ID == x.Key),
                    Count = x.Value.Item2
                }).ToList()
            };
        }

        public void Update(ProductBindingModel model)
        {
            Name = model.Name;
            Price = model.Price;
        }

        public ProductViewModel GetViewModel => new()
        {
            ID = ID,
            Name = Name,
            Price = Price,
            EmployeeID = EmployeeID,
            EmployeeUsername = Employee.Username,
            ProductComponents = ProductComponents
        };

        public void UpdateComponents(ComputerStoreDatabase context, ProductBindingModel model)
        {
            var productComponents = context.ProductComponents.Where(rec => rec.ProductID == model.ID).ToList();
            if (productComponents != null && productComponents.Count > 0)
            {
                if (productComponents.Where(rec => !model.ProductComponents.ContainsKey(rec.ComponentID)).Any())
                {
                    context.ProductComponents.RemoveRange(productComponents.Where(rec => !model.ProductComponents.ContainsKey(rec.ComponentID)));
                    context.SaveChanges();
                }
                foreach (var updateComponent in productComponents.Where(x => model.ProductComponents.ContainsKey(x.ComponentID)))
                {
                    updateComponent.Count = model.ProductComponents[updateComponent.ComponentID].Item2;
                    model.ProductComponents.Remove(updateComponent.ComponentID);
                }
                context.SaveChanges();
            }

            var product = context.Products.First(x => x.ID == ID);
            foreach (var pc in model.ProductComponents)
            {
                context.ProductComponents.Add(new ProductComponent
                {
                    Product = product,
                    Component = context.Components.First(x => x.ID == pc.Key),
                    Count = pc.Value.Item2
                });
                context.SaveChanges();
            }
            _productComponents = null;
        }
        
    }
}
